
package tiny;

/*
Palavras reservadas:
 if, then, else, end, repeat, until, read, write, var,
 array, record, of, true, false, nil,
 int, bool, char.

Operadores: 
  +, -, *, /, =, <, (, ), ;, :=, ., :, [, ].

Nmeros:
 um ou mais dgitos. -> [0-9]+
 
Identificadores: 
 letras seguidas de letras, nmeros ou _.
   -> [a-zA-Z][a-zA-Z0-9_]*
   
Comentrios: 
 cercados por chaves ({...}), no podem ser aninhados.  
     -> {[^}]*}

Espao em branco:
  -> [ \r\n\t]+

 * 
 * 
 */

%%

%class JFScanner
%function getToken
%type Token

%line
%column

%{
  int col;
%}

%%

<YYINITIAL> {

if         { return new Token(Token.IF, yyline, yycolumn); }

then       { return new Token(Token.THEN, yyline, yycolumn); }

else       { return new Token(Token.ELSE, yyline, yycolumn); }

end        { return new Token(Token.END, yyline, yycolumn); }

repeat     { return new Token(Token.REPEAT, yyline, yycolumn); }

until      { return new Token(Token.UNTIL, yyline, yycolumn); }

read       { return new Token(Token.READ, yyline, yycolumn); }

write      { return new Token(Token.WRITE, yyline, yycolumn); }

var        { return new Token(Token.VAR, yyline, yycolumn); }

array      { return new Token(Token.ARRAY, yyline, yycolumn); }

record     { return new Token(Token.RECORD, yyline, yycolumn); }

of         { return new Token(Token.OF, yyline, yycolumn); }

true       { return new Token(Token.TRUE, yyline, yycolumn); }

false      { return new Token(Token.FALSE, yyline, yycolumn); }

nil        { return new Token(Token.NIL, yyline, yycolumn); }

int        { return new Token(Token.INT, yyline, yycolumn); }

bool       { return new Token(Token.BOOL, yyline, yycolumn); }

function   { return new Token(Token.FUNCTION, yyline, yycolumn); }

begin      { return new Token(Token.BEGIN, yyline, yycolumn); }

return     { return new Token(Token.RETURN, yyline, yycolumn); }

[+\-*/=<();,.\[\]:\^@] { return new Token(yytext().charAt(0),
                                   yyline, yycolumn); }
 
:=         { return new Token(Token.ASSIGN, yyline, yycolumn); } 

[0-9]+     { return new Token(Token.NUM, 
                              Integer.parseInt(yytext()),
                              yyline, yycolumn); }
                              
[a-z][a-zA-Z0-9_]*  { return new Token(Token.ID, yytext(),
                                       yyline, yycolumn); }
                                       
"{"~"}"     { }

[ \r\n\t]+  { }                                       

<<EOF>>     { return new Token(Token.EOF); }

.           { throw new Error("caractere no identificado: " + yytext()); }


}

                